%
% Este ficheiro contem um texto em Literate Haskell
%    Partes do ficheiro so texto em LaTeX e outras em Haskell
%    
%  Advanced Functional Programming
%  Universidade do Minho
%  2003/2004
%


\documentclass[12pt]{article}
\usepackage{a4wide}
\usepackage{graphics}
\usepackage{graphicx}
\usepackage{color}
\usepackage{alltt}
\usepackage[portuges]{babel}


\usepackage[latin1]{inputenc}
\usepackage{latexsym}


\usepackage[dvips]{epsfig}
\usepackage{epic}
\usepackage{eepic}


\def\Ipe#1{\def\IPEfile{#1}\input{#1}}

\newenvironment{code}
{\medskip\par\noindent\textbf{Haskell Code} \hspace{1cm} \hrulefill \\ 
\nopagebreak\smallskip\nopagebreak
\begin{center}
\begin{minipage}{.80\textwidth} 
\begin{alltt}\small}
{\end{alltt}
\end{minipage}
\end{center}
\hrule\medskip\noindent
}


\newtheorem{exercicio}{Exercise}[section]

\title{\sf\huge Advanced Functional Programming \\
\begin{tabular}{c}
{\small LMCC \& LESI}, {\small Universidade do Minho} \\
Strafunski: Exercises \\ 
\end{tabular}
}

\author{}
\date{}


%-------------------- Inicio do Documento -------------------------------

\begin{document}

\maketitle


\section{Refactoring of Haskell Programs}


One of the examples included in the Strafunski distribution is a
program \verb@HsTransform@ that transforms haskell source code. It
performs two simple transformations: the elimination of the do
constructor and the introduction of a new type definition. You can run
the program (in hugs) as follows:

\begin{alltt} \footnotesize
runhugs  +o -98 +N
 -P<StrategyLib>/examples/haskell:<StrategyLib>/library:<StrategyLib>/models/drift-default: 
 HsTransform <inputfile> <outputfile>
\end{alltt}

\noindent
where \verb+StrategyLib+ is the location where you installed
Strafunski's StrategyLib (version 4.0-beta), and \verb+inputfile+ and
\verb+outputfile+ are the names of the Haskell program to be
transformed, and the file where you want the transformed program to be
written to.


\begin{exercicio} 

Write a little test program that makes use of various \verb+do+
constructions. Run the transformation program on your test program and
inspect the resulting program. Run your test program as well as the
transformed program to see if they behave the same.

Note: Be sure to test nested do-expression and monadic let-expressions
inside do's.

\end{exercicio}

\noindent
For instance, consider the following example Haskell program:

\begin{code}
Put your input test module here
\end{code}

Running the \verb+HsTransform+ program on it produces the following output:

\begin{code}
Put the transformed module here
\end{code}

\noindent
The elimination of \verb+do+ expressions is defined in the Haskell
Report, in section 3.14. This transformation is expressed in
Strafunski as follows:


\begin{code}
doElim 	   :: (Term a, MonadPlus m) => a -> m a
doElim h   =  applyTP (innermost (monoTP doElim1)) h

doElim1    :: MonadPlus m => HsExp -> m HsExp
doElim1 (HsDo [HsQualifier e]) 	
  = return e
doElim1 (HsDo (HsQualifier e:stmts))	
  = return (HsInfixApp e (HsVar (hsSymbol ">>")) (HsDo stmts))
doElim1 (HsDo (HsGenerator p e:stmts))
  = do ok <- new_name
       return (letPattern ok p e stmts)
doElim1 (HsDo (HsLetStmt decls:stmts))
  = return (HsLet decls (HsDo stmts))
doElim1 _                      
  = mzero
\end{code}


The function \verb+doElim1+ implements a single \verb+do+-elimination
rewrite step. It follows the definition in the Haskell Report quite
closely, except that instead of Haskell's concrete surface syntax,
it's abstract syntax is used.

The function \verb+doElim+ completes the rewrite step \verb+doElim1+
into a complete transformation. The \verb+monoTP+ combinator lifts the
rewrite step to a \emph{generic} rewrite step, and the
\verb+innermost+ combinator applies this generic rewrite step
repeatedly according to the innermost traversal strategy, until a
fixpoint is reached (no more redexes remain).




\begin{exercicio} 

In section 3.6 of the Haskell report a translation of the
\verb@if_then_else@ construct is presented. Implement this translation
in a strategic function named \verb@ifElim@. In order to incorporate
this transformation in the distributed package, just modify the Main
module to import module \verb+IfElim+, and modify one line in the main
function into:

\begin{verbatim}
      iowrap (mpipe [doElim,data2newtypeConv,ifElim])
\end{verbatim}

\end{exercicio}

\noindent

\noindent
For instance, consider the following example Haskell program:

\begin{code}
module TestIfElim where
f a = if (a > 1)
      then a
      else -a
\end{code}

Running the \verb+HsTransform+ program on it produces the following output:

\begin{code}
module TestIfElim where
f a
  = case (a > 1) of
	True -> a
	False -> - a
\end{code}


This transformation is expressed in Strafunski as follows:

\begin{code}
module IfElim where

import Datatypes
import DatatypesTermInstances
import StrategyLib
import Monad

ifElim 	   :: (Term a, MonadPlus m) => a -> m a
ifElim h   =  ...

ifElim1    :: MonadPlus m => HsExp -> m HsExp

\end{code}


\begin{exercicio} 

Implement the list comprehension elimination according to the rules
defined in section 3.11 of the haskell report. (if you solve this
exercise please email Joost the solution to be included in the next
Strafunski distribution).

\end{exercicio}


\section{Querying and transforming XML Documents}

Using the XML support from the HaXml project, you can easily import XML
documents into Haskell, and export them from Haskell. If you can supply a DTD
for your documents, you can additionally, inside Haskell, marshall between a
generic representation of XML documents (verified only) and a typed Haskell
representation of XML documents (validated by the Haskell type system).

As a running example, we will use documents that contain course descriptions.
An example document is given in \verb+teste.xml+, and its DTD is given in
\verb+Curso.dtd+.

To get things running you need to run \verb+DtdToHaskell+ on the given DTD to
obtain a file \verb+Curso.hs+ containing the Haskell datatypes that will
represent your document. Additionally, this file will contain instances of the
\verb+XmlContent+ class that provides the marshalling functionality.

\begin{exercicio} 
Run \verb+DtdToHaskell+ and inspect the content of the resulting file. Try to
understand the relationship between the DTD and the Haskell types, and between
XML documents and Haskell terms.
\end{exercicio}

\noindent
After generating the code, we can define some wrapper functions for performing
queries and transformations on given XML documents:

\begin{code}
module XMLExercise where

import Curso
import Text.XML.HaXml.Parse (xmlParse)
import Text.XML.HaXml.Types (Document(..),Content(..))
import Text.XML.HaXml.Xml2Haskell (showXml,fromElem,XmlContent(..))

processQ q xmlFile
  = do xmlInput <- readFile xmlFile
       let curso = fromString xmlInput :: Curso
       result <- q curso
       putStrLn (show result)

processT t xmlFile
  = do xmlInput <- readFile xmlFile
       let curso = fromString xmlInput :: Curso
       result <- t curso
       putStrLn (showXml result)

-- Helper for parsing

fromString :: XmlContent a => String -> a
fromString s 
  = forceResult (fst (fromElem [getElem (xmlParse "hoi" s)]))
    where
      getElem (Document _ _ e) = CElem e
      forceResult Nothing  = error "XML PARSE ERROR"
      forceResult (Just r) = r
\end{code}


\begin{exercicio}
Run a minimal query and a minimal transformation on the given example document
\verb+teste.xml+ to see if everything is working properly. Use \verb+ghci+ as
follows:
\begin{verbatim}
ghci -cpp -fglasgow-exts -fallow-overlapping-instances\
 -fallow-undecidable-instances -package data \
  -i<StrategyLib>/library:<StrategyLib>/models/drift-default:
  -i<HaXml>/src: XMLExercise
\end{verbatim}
where \verb+<HaXml>+ is the place where you installed HaXml. 
At the prompt, try:
\begin{verbatim}
*Main> processQ (const . putStrLn $ "Nada") "teste.xml"
*Main> processT return "teste.xml"
\end{verbatim}
Make sure you understand what is going on. For instance, do you understand why
the type annotation (\verb+:: Curso+) is used?
\end{exercicio}

If we make use of HaXml's marshalling functionality, writing queries and
transformations on XML documents can be done by plain functional programming on
the Haskell terms that represent these documents. But, since these documents
can contain large numbers of different element tags and attributes, plain
functional programming may quickly turn out to be clumsy.

\begin{exercicio}
Program a query that extracts all student names from a given XML document,
using standard functional programming techniques. Run the query with
\verb+processQ+.
\end{exercicio}

The following code shows how we can extract student names from a given XML
document using a set of recursive functions:

\begin{code}
studentNames
 = processQ (return . qCurso) "teste.xml"
   where
     qCurso (Curso _ _ cadeiras) 
       = ...
     qCadeira (Cadeira _ _ _ inscritos _) 
       = ...
     qInscritos (Inscritos _ alunos)
       = ...
     qAluno (Aluno _ nome)
       = nome
\end{code}

\noindent
Note that the recursive functions do pattern matching on all the parent types
of \verb+Aluno+. If the definition of \verb+Curso+, \verb+Cadeira+, or
\verb+Inscritos+ is changed (for instance by allowing further elements inside
them), the code above will need to be adapted as well.

Using the strategic function combinators of Strafunski, we can improve on this
situation. To use Strafunski, we need to generate instances of the \verb+Term+
class from the datatypes in \verb+Curso.hs+, using \verb+DrIFT+.

\begin{exercicio}
Generate the file \verb+CursoTermInstances.hs+ with an invocation of
\verb+DrIFT+, as follows:
\begin{verbatim}
> echo "module CursoTermInstances where" > CursoTermInstances.hs
> echo "import Curso" >> CursoTermInstances.hs
> echo "import TermRep" >> CursoTermInstances.hs
> DrIFT -g Term -g Typeable -r Curso.hs >> CursoTermInstances.hs
\end{verbatim}
The instances of class \verb+Term+ and \verb+Typeable+ are used only internally
by the strategy combinators of Strafunski. As a user of these combinators, you
do not need to understand them.
\end{exercicio}

Using Strafunski, the same query can be implemented more concisely as follows:

\begin{code}
...
import StrategyLib
import CursoTermInstances
...

studentNames'
  = processQ (applyTU q) "teste.xml"
    where
      q = stop_tdTU (monoTU qAluno) 
      qAluno (Aluno _ nome) = return [nome]
\end{code}

\noindent The traversal strategy \verb+stop_tdTU+ is used to search for
students in a top down fashion, without continuing the search on the
subelements of a student. The \verb+monoTU+ combinator turns the monomorphic
function \verb+qAluno+ into a generic function that succeeds on students, but
fails on other elements.

\begin{exercicio}
Verify that the reformulated query produces the same results as the original
one. Make sure you understand the reformulated query.
\end{exercicio}

Apart from querying the document, we can transform it. For instance, we can
update the attribute \verb+n+ of the element \verb+Inscritos+ such that we are
sure that it matches the actual number of students registred for the course.

\begin{exercicio}
Complete the following code. And run it on the document \verb+teste.xml+.
\end{exercicio}

The following code traverses the document to find elements of type
\verb+Inscritos+. These elements are subsequently queried to find the number of
students that it holds. Finally, the attribute of the element \verb+Inscritos+
is updated to the computed number of students.

\begin{code}
countAlunos 
 = applyTU ( ... )
   where
     countAluno (a::Aluno) = return (1::Int)

updateInscritosN
  = applyTP( ... )
    where
      update (Inscritos attrs alunos)
        = do n <- countAlunos alunos
             let attrs' = attrs{inscritosN=Just (show n)}
             return (Inscritos attrs' alunos)
\end{code}

\noindent
Note the use of record update to perform attribute updating.

\end{document}
